# Grégory Coutable 2022
# Partage sous licence Creative Commons :
# https://creativecommons.org/licenses/by-sa/3.0/fr/
#
# Version 1

import pygame
from pygame.locals import *
#pour le rendre dispo de n'importe où
import os
pathname = os.path.dirname(__file__)

class GUIreversi (object):
    
    def __init__(self):
        """
        Initialise une fenètre du jeu reversi (ou Othello en version commerciale) et l'affiche totalement vide.
        Cette classe ne contient aucun attribut publique.
        """
        
        #initialisation de pygame
        pygame.init()
        
        self.wPiece = 80
        self.wTapis = 9 * self.wPiece
        self.hTapis = 9 * self.wPiece
        self.hBandeau = 100
        self.rectBandeau = pygame.Rect(0, self.hTapis, self.wTapis, self.hBandeau)
        
        #on définit la fenêtre de base de notre jeu
        self.fenetre = pygame.display.set_mode((self.wTapis, self.hTapis + self.hBandeau))
        #un titre sur cette fenêtre
        pygame.display.set_caption("Reversi")
        #Chargement et collage du fond
        self.plateau = pygame.Surface((8 * self.wPiece, 8 * self.wPiece))
        
        self.refresh([[0 for x in range(8)] for y in range(8)])

        
       #circle(surface, color, center, radius)
        

    def refresh(self, g, t = ""):
        """
        Cette méthode rafraichie l'affichage du jeu Reversi conformément à la grille passée en argument.
        
        g est une grille : une liste de 8 listes de 8 éléments par exemple, ou n'importe quel autre objet indexable.
        
            - g[0][0] est la case en haut à gauche
            - g[7][7] est la case en bas à droite
            - g[0][7] est la case en haut à droite
        
        t est un texte à afficher à destination des joueurs.
        """
        self.plateau.fill(0x50c63d)
        for u in range(1, 8):
            pygame.draw.line(self.plateau, '0x9a5e2a', (0, u * self.wPiece), (720, u * self.wPiece), 3)
            pygame.draw.line(self.plateau, '0x9a5e2a', (u * self.wPiece, 0), (u * self.wPiece, 720), 3)
            
        for y in range(8):
            for x in range(8):
                if g[y][x] == 1:
                    pygame.draw.circle(self.plateau, '0x2f7235', (x * self.wPiece + self.wPiece // 2 + 3, y * self.wPiece + self.wPiece // 2 + 3), self.wPiece // 2 - 10)    
                    pygame.draw.circle(self.plateau, '0xE5E5E5', (x * self.wPiece + self.wPiece // 2, y * self.wPiece + self.wPiece // 2), self.wPiece // 2 - 10)    
                elif g[y][x] == 2:
                    pygame.draw.circle(self.plateau, '0x2f7235', (x * self.wPiece + self.wPiece // 2 + 3, y * self.wPiece + self.wPiece // 2 + 3), self.wPiece // 2 - 10)    
                    pygame.draw.circle(self.plateau, 'black', (x * self.wPiece + self.wPiece // 2, y * self.wPiece + self.wPiece // 2), self.wPiece // 2 - 10)    


        self.fenetre.fill(0x9a5e2a)
        self.fenetre.blit(self.plateau, (self.wPiece // 2, self.wPiece // 2))
        
        police = pygame.font.SysFont("Arial Black.ttf", 36)
        recPos = pygame.Rect(self.wPiece // 2, 0, self.wPiece, self.wPiece // 2)
        for i in range(8):
            txt = police.render(chr(65 + i), True, 'white')
            txtRect = txt.get_rect()
            recPos.top = 0
            txtRect.center = recPos.center
            self.fenetre.blit(txt, txtRect)
            recPos.bottom = self.hTapis
            txtRect.center = recPos.center
            self.fenetre.blit(txt, txtRect)
            
            recPos.left = recPos.right

        recPos = pygame.Rect(0, self.wPiece // 2, self.wPiece // 2, self.wPiece)
        for i in range(8):
            txt = police.render(str(i + 1), True, 'white')
            txtRect = txt.get_rect()
            recPos.left = 0
            txtRect.center = recPos.center
            self.fenetre.blit(txt, txtRect)
            recPos.right = self.wTapis
            txtRect.center = recPos.center
            self.fenetre.blit(txt, txtRect)
            
            recPos.top = recPos.bottom

        #zone de texte
        #Une police
        police = pygame.font.SysFont("Arial Black",28)
        #un texte
        texte = police.render(t, True, pygame.Color("#FFFF00"))
        #pour centrer le texte
        rectTexte = texte.get_rect()
        rectTexte.center = self.rectBandeau.center
        self.fenetre.blit(texte, rectTexte)
        #Rafraîchissement de l'écran
        pygame.display.update()

    def messCentre(self, mess):
        """
        Cette méthode permet d'afficher un message plein écran.
        """
        police = pygame.font.Font(os.path.join(pathname, "led.ttf"),80)
        texte = police.render(mess, True, pygame.Color("yellow"))
        #pour centrer le texte
        rectTexte = texte.get_rect()
        rectTexte.center = self.fenetre.get_rect().center
        self.fenetre.blit(texte,rectTexte)
        #Rafraîchissement de l'écran
        pygame.display.update()

    def waitClick(self):
        """
        Cette méthode attend l'action d'un joueur. Elle gère trois type d'actions :
            - demande fermeture de la fenètre : fermeture propre de la fenètre pygame et fin du programme python.
            - click sur la fenetre : retourne  un tuple qui référence la case cliquée (x, y).
            - appui sur des touches (uniquement y, n, s, r, c, d, fleche droite : retourne la lettre correspondante.
        Une fois exécutée, on ne peut sortir de cette méthode que par l'une de ces trois actions.
        """
        while True:
            #Limitation de vitesse de la boucle
            pygame.time.Clock().tick(30)
            
            for event in pygame.event.get():    #Attente des événements
                if event.type == QUIT:
                    pygame.quit()
                    exit()
                    
                if event.type == MOUSEBUTTONDOWN:
                    if event.button == 1:   #Si clic gauche
                        if 0.5 * self.wPiece < event.pos[0] < self.wPiece * 8.5 and 0.5 * self.wPiece < event.pos[1] < self.wPiece * 8.5:
                            return ((event.pos[0] - self.wPiece // 2)//(self.wPiece), (event.pos[1] - self.wPiece // 2)//(self.wPiece))                    
                    
                if event.type == KEYDOWN:
                    touches = {K_RIGHT : '_R', K_LEFT : '_L', K_UP : '_U', K_DOWN : '_D', K_RETURN : '_E', K_BACKSPACE : '_B', K_ESCAPE : '_S'}
                    touche = event.key
                    if touche in touches:
                        return touches[touche]
                    return event.unicode  
                    
                         
if __name__ == "__main__":
    import time
   
    j = GUIreversi()
    
    time.sleep(2)
    
    g = [[(x + y)%3 for x in range(8)] for y in range(8)]
    j.refresh(g, "A vous de jouer !!")
    time.sleep(2)
    j.messCentre('GAME OVER')
    

    
    while True :
        print(j.waitClick())

